def mask_missing(arr, values_to_mask): ...
def clean_fill_method(method, allow_nearest: bool = ...): ...
def clean_interp_method(method, **kwargs): ...
def interpolate_2d(
    values, method: str = ..., axis: int = ..., limit=..., fill_value=..., dtype=...
): ...
def get_fill_func(method): ...
def clean_reindex_fill_method(method): ...
