class EngFormatter:
    ENG_PREFIXES = ...
    accuracy = ...
    use_eng_prefix = ...
    def __init__(
        self, accuracy: int | None = ..., use_eng_prefix: bool = ...
    ) -> None: ...
    def __call__(self, num: float) -> str: ...

def set_eng_float_format(accuracy: int = ..., use_eng_prefix: bool = ...) -> None: ...
