from pandas.core.arrays.sparse.accessor import (
    SparseAccessor as SparseAccessor,
    SparseFrameAccessor as SparseFrameAccessor,
)
from pandas.core.arrays.sparse.array import SparseArray as SparseArray
from pandas.core.arrays.sparse.dtype import SparseDtype as SparseDtype
